"""Test functions for datetime_utils.py.
"""
from datetime import datetime

from aracnid_utils.datetime_utils import isoweek, fromisoweek

# initialize module variables
REF_ISO_WEEK = '2020-W25'
REF_WEEK_DATE1 = '2020-06-15T00:00:00-04:00'
REF_WEEK_DATE2 = '2020-06-21T23:59:59-04:00'
REF_WEEK_DATE3 = '2020-06-22T00:00:00-04:00'

def test_get_isoweek_from_date_start():
    """Tests isoweek() given the earliest possible date in the week.
    """
    week_date = datetime.fromisoformat(REF_WEEK_DATE1)
    week_str = isoweek(week_date)
    assert week_str == REF_ISO_WEEK

def test_get_isoweek_from_date_end():
    """Tests isoweek() given the latest possible date in the week.
    """
    week_date = datetime.fromisoformat(REF_WEEK_DATE2)
    week_str = isoweek(week_date)
    assert week_str == REF_ISO_WEEK

def test_get_isoweek_from_date_next_week():
    """Tests isoweek() given a date in the following week.
    """
    week_date = datetime.fromisoformat(REF_WEEK_DATE3)
    week_str = isoweek(week_date)
    assert week_str != REF_ISO_WEEK

def test_get_date_from_isoweek():
    """Tests fromisoweek() given a standard ISO Week date string.
    """
    week_date = fromisoweek(REF_ISO_WEEK)
    assert week_date.isoformat() == REF_WEEK_DATE1



"""Provides additional datetime functions.
"""
import re

from datetime import datetime, timedelta
from dateutil import tz
from dateutil.parser import parse

START_MIN = datetime(2000, 1, 1, 0, 0).astimezone()

def isoweek(datetime_obj):
    """Calculates the ISO Week from the specified datetime object.

    Args:
        datetime_obj: Datetime object.

    Returns:
        The ISO Week.
    """
    year, week, _ = datetime_obj.isocalendar()
    return f'{year}-W{week:02}'

def fromisoweek(week_string):
    """Calculates the datetime from the specified ISO Week string.

    Args:
        week_string: The ISO week string.

    Returns:
        The datetime object.
    """
    pattern_isoweek = r'^\d{4}-W\d{2}$'

    if re.search(pattern_isoweek, week_string):
        year = int(week_string[:4])
        week = int(week_string[-2:])
        return datetime.fromisocalendar(year, week, 1).astimezone()

    return None

def timespan(**kwargs):
    """Calculates the endpoints of a timespan.

    The timespan is specified by a set of keyword arguments. Separate arguments
    are used to provide datetime objects or strings representing datetimes.
    The strings can be any parsable notation that results in a datetime object.
    This includes ISO Week Notation, e.g., "2020-W13".

    If the begin time is not specified, the minimum start time is used.
    If the thru time is not specified, the current time is used.
    If a week string is specified, the start time and the thru time will default
    to the week. The week string can be used in conjunction with either begin
    time or thru time.

    Args:
        begin: Datetime object representing the start of the timespan.
        begin_str: String representing the start of the timespan.
        thru: Datetime object representing the end of the timespan.
        thru_str: String representing the end of the timespan.
        week_str: String representing a single week.

    Returns:
        The start and end datetime objects that define the timespan.
    """
    begin_dt = kwargs.get('begin')
    thru_dt = kwargs.get('thru')
    begin_str = kwargs.get('begin_str') or kwargs.get('week_str')
    thru_str = kwargs.get('thru_str') or kwargs.get('week_str')

    # initialize week notation pattern
    pattern_isoweek = r'^\d{4}-W\d{2}$'

    # convert string arguments
    if not begin_dt and begin_str:
        # check iso week notation
        if re.search(pattern_isoweek, begin_str):
            year_num = int(begin_str[:4])
            week_num = int(begin_str[-2:])
            begin_dt = datetime.fromisocalendar(
                year_num, week_num, 1).astimezone()

        else:
            begin_dt = parse(begin_str).astimezone()
    if not thru_dt and thru_str:
        # check iso week notation
        if re.search(pattern_isoweek, thru_str):
            year_num = int(thru_str[:4])
            week_num = int(thru_str[-2:])
            thru_dt = datetime.fromisocalendar(
                year_num, week_num, 1).astimezone()
            thru_dt += timedelta(days=7)

        else:
            thru_dt = parse(thru_str).astimezone()

    # check missing bounds
    if not begin_dt:
        begin_dt = START_MIN
    if not thru_dt:
        thru_dt = datetime.now(tz.tzlocal())

    return begin_dt, thru_dt


if __name__ == '__main__':
    pass



"""Test functions for datetime_utils.py.
"""
from datetime import datetime, timedelta

from dateutil import tz

from aracnid_utils.datetime_utils import timespan

# initialize module variables
REF_BEGIN_STR = '2020-06-01T00:00:00-04:00'
REF_THRU_STR = '2020-06-08T00:00:00-04:00'
REF_BEGIN_ISO = '2020-W10'
REF_THRU_ISO = '2020-W25'
REF_WEEK_STR = '2020-W23'


def test_timespan_args_begin_str_and_thru_str():
    """Tests timespan arguments: begin_str, thru_str.
    """
    start, end = timespan(begin_str=REF_BEGIN_STR, thru_str=REF_THRU_STR)

    assert isinstance(start, datetime)
    assert start.isoformat() == REF_BEGIN_STR

    assert isinstance(end, datetime)
    assert end.isoformat() == REF_THRU_STR

def test_timespan_args_begin_str_and_thru_str_none():
    """Tests timespan arguments: begin_str, thru_str=None.
    """
    start, end = timespan(begin_str=REF_BEGIN_STR, thru_str=None)
    end_now = datetime.now(tz.tzlocal())

    assert isinstance(start, datetime)
    assert start.isoformat() == REF_BEGIN_STR

    assert isinstance(end, datetime)
    assert end.isoformat()[0:18] == end_now.isoformat()[0:18]

def test_timespan_args_begin_str_and_thru_str_missing():
    """Tests timespan arguments: begin_str, thru_str missing.
    """
    start, end = timespan(begin_str=REF_BEGIN_STR)
    end_now = datetime.now(tz.tzlocal())

    assert isinstance(start, datetime)
    assert start.isoformat() == REF_BEGIN_STR

    assert isinstance(end, datetime)
    assert end.isoformat()[0:18] == end_now.isoformat()[0:18]

def test_timespan_args_begin_str_none_and_thru_str():
    """Tests timespan arguments: begin_str=None, thru_str.
    """
    start, end = timespan(begin_str=None, thru_str=REF_THRU_STR)
    start_first = datetime(2000, 1, 1, 0, 0).astimezone()

    assert isinstance(start, datetime)
    assert start.isoformat() == start_first.isoformat()

    assert isinstance(end, datetime)
    assert end.isoformat() == REF_THRU_STR

def test_timespan_args_begin_str_missing_and_thru_str():
    """Tests timespan arguments, begin_str missing, thru_str.
    """
    start, end = timespan(thru_str=REF_THRU_STR)
    start_first = datetime(2000, 1, 1, 0, 0).astimezone()

    assert isinstance(start, datetime)
    assert start.isoformat() == start_first.isoformat()

    assert isinstance(end, datetime)
    assert end.isoformat() == REF_THRU_STR

def test_timespan_args_begin_and_thru():
    """Tests timespan arguments: begin, thru.
    """
    begin = datetime.fromisoformat(REF_BEGIN_STR)
    thru = datetime.fromisoformat(REF_THRU_STR)
    start, end = timespan(begin=begin, thru=thru)

    assert isinstance(start, datetime)
    assert start == begin

    assert isinstance(end, datetime)
    assert end == thru

def test_timespan_args_begin_and_thru_none():
    """Tests timespan arguments: begin, thru=None.
    """
    begin = datetime.fromisoformat(REF_BEGIN_STR)
    start, end = timespan(begin=begin, thru=None)
    thru_now = datetime.now(tz.tzlocal())

    assert isinstance(start, datetime)
    assert start == begin

    assert isinstance(end, datetime)
    assert end.isoformat()[0:18] == thru_now.isoformat()[0:18]

def test_timespan_args_begin_and_thru_missing():
    """Tests timespan arguments: begin, thru missing.
    """
    begin = datetime.fromisoformat(REF_BEGIN_STR)
    start, end = timespan(begin=begin)
    thru_now = datetime.now(tz.tzlocal())

    assert isinstance(start, datetime)
    assert start == begin

    assert isinstance(end, datetime)
    assert end.isoformat()[0:18] == thru_now.isoformat()[0:18]

def test_timespan_args_begin_none_and_thru():
    """Tests timespan arguments: begin=None, thru.
    """
    thru = datetime.fromisoformat(REF_THRU_STR)
    start, end = timespan(begin=None, thru=thru)
    start_first = datetime(2000, 1, 1, 0, 0).astimezone()

    assert isinstance(start, datetime)
    assert start == start_first

    assert isinstance(end, datetime)
    assert end == thru

def test_timespan_args_begin_missing_and_thru():
    """Tests timespan arguments: begin missing, thru.
    """
    thru = datetime.fromisoformat(REF_THRU_STR)
    start, end = timespan(thru=thru)
    start_first = datetime(2000, 1, 1, 0, 0).astimezone()

    assert isinstance(start, datetime)
    assert start == start_first

    assert isinstance(end, datetime)
    assert end == thru

def test_timespan_args_begin_iso_week_and_thru_iso_week():
    """Tests timespan arguments: begin iso week, thru iso week.
    """
    start, end = timespan(begin_str=REF_BEGIN_ISO, thru_str=REF_THRU_ISO)
    begin = datetime.fromisocalendar(2020, 10, 1).astimezone()
    thru = datetime.fromisocalendar(2020, 25, 1).astimezone() + timedelta(days=7)

    assert isinstance(start, datetime)
    assert start == begin

    assert isinstance(end, datetime)
    assert end == thru

def test_timespan_args_begin_iso_week_and_thru_str_none():
    """Tests timespan arguments: begin iso week, thru_str=None.
    """
    start, end = timespan(begin_str=REF_BEGIN_ISO, thru_str=None)
    begin = datetime.fromisocalendar(2020, 10, 1).astimezone()
    end_now = datetime.now(tz.tzlocal())

    assert isinstance(start, datetime)
    assert start == begin

    assert isinstance(end, datetime)
    assert end.isoformat()[0:18] == end_now.isoformat()[0:18]

def test_timespan_args_begin_iso_week_and_thru_str_missing():
    """Tests timespan arguments: begin iso week, thru_str missing.
    """
    start, end = timespan(begin_str=REF_BEGIN_ISO)
    begin = datetime.fromisocalendar(2020, 10, 1).astimezone()
    end_now = datetime.now(tz.tzlocal())

    assert isinstance(start, datetime)
    assert start == begin

    assert isinstance(end, datetime)
    assert end.isoformat()[0:18] == end_now.isoformat()[0:18]

def test_timespan_args_begin_str_none_and_thru_iso_week():
    """Tests timespan arguments: begin_str=None, thru iso week.
    """
    start, end = timespan(begin_str=None, thru_str=REF_THRU_ISO)
    start_first = datetime(2000, 1, 1, 0, 0).astimezone()
    thru = datetime.fromisocalendar(2020, 25, 1).astimezone() + timedelta(days=7)

    assert isinstance(start, datetime)
    assert start.isoformat() == start_first.isoformat()

    assert isinstance(end, datetime)
    assert end == thru

def test_timespan_args_begin_str_missing_and_thru_iso_week():
    """Tests timespan arguments: begin_str missing, thru iso week.
    """
    start, end = timespan(thru_str=REF_THRU_ISO)
    start_first = datetime(2000, 1, 1, 0, 0).astimezone()
    thru = datetime.fromisocalendar(2020, 25, 1).astimezone() + timedelta(days=7)

    assert isinstance(start, datetime)
    assert start.isoformat() == start_first.isoformat()

    assert isinstance(end, datetime)
    assert end == thru

def test_timespan_args_week_str():
    """Tests timespan argument: week_str.
    """
    start, end = timespan(week_str=REF_WEEK_STR)

    assert isinstance(start, datetime)
    assert start.isoformat() == REF_BEGIN_STR

    assert isinstance(end, datetime)
    assert end.isoformat() == REF_THRU_STR

def test_timespan_args_week_str_and_begin_str():
    """Tests timespan arguments: week_str, begin_str.
    """
    start, end = timespan(week_str=REF_THRU_ISO, begin_str=REF_BEGIN_STR)
    thru = datetime.fromisocalendar(2020, 25, 1).astimezone() + timedelta(days=7)

    assert isinstance(start, datetime)
    assert start.isoformat() == REF_BEGIN_STR

    assert isinstance(end, datetime)
    assert end.isoformat() == thru.isoformat()

def test_timespan_args_week_str_and_thru_str():
    """Tests timespan arguments: week_str, thru_str.
    """
    start, end = timespan(week_str=REF_BEGIN_ISO, thru_str=REF_THRU_STR)
    begin = datetime.fromisocalendar(2020, 10, 1).astimezone()

    assert isinstance(start, datetime)
    assert start.isoformat() == begin.isoformat()

    assert isinstance(end, datetime)
    assert end.isoformat() == REF_THRU_STR

def test_timespan_args_week_str_and_begin_str_and_thru_str():
    """Tests timespan arguments: week_str, begin_str, thru_str.

    The week string is ignored in this case.
    """
    start, end = timespan(week_str=REF_WEEK_STR, begin_str=REF_BEGIN_STR, thru_str=REF_THRU_STR)

    assert isinstance(start, datetime)
    assert start.isoformat() == REF_BEGIN_STR

    assert isinstance(end, datetime)
    assert end.isoformat() == REF_THRU_STR



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['aracnid_utils']

package_data = \
{'': ['*']}

install_requires = \
['aracnid-logger', 'python-dateutil ~=2.8']

setup(name='aracnid_utils',
      version='1.0.0a4',
      description='This package contains utilities that I use frequently.',
      author='Jason Romano',
      author_email='aracnid@gmail.com',
      url='https://github.com/aracnid/aracnid-utils',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      python_requires='~=3.8',
     )
